#include "macro-lib.cc"

bool doingSeeds = false;

Point menuCorner = { 174, 100 };
Point pinMenu = { menuCorner.x - 19, 18 };
Point pinClose = pinMenu;
Point menuOption = { 30, 50 };
Point seedPoint = { 10, 45 };
double matchSeed[] = { 0, 0, 0, 0.01 };
Point titlePoint = { 10, 14 };
double matchTitle[] = { 0, 0, 0, 0.01 };
Point scrollBorder = { 148, 38 };
double matchScrollBorder[] = { 0.22, 0.20, 0.16, 0.03 };
int maxSeeds = 5;
Point menuDrag = { 0, menuCorner.y / 2 };
Point plantMenu;
Point flaxClick;

void pinFlaxAt(Point p)
{
  clickWait(plantMenu);
  waitForRefresh();
  clickWait(flaxClick);
  clickWait(flaxClick - menuDrag + pinMenu);
  drag(flaxClick, p + menuDrag);
  waitForRefresh();
}

struct FlaxMenu {
  Point loc;
  int numActions;
  time_t lastAction;
  int numSeeds;
  bool hadSeeds;
  bool done;

  FlaxMenu(Point loc) {
    this->loc = loc;
    numActions = 0;
    lastAction = time(0);
    numSeeds = 0;
    hadSeeds = false;
    done = false;
    pinFlaxAt(loc);
  }

  void doMenu() {
    if (done) return;
    clickWait(loc + menuOption);
    numActions++;
    lastAction = time(0);
  }

  void unPin() {
    clickWait(loc + pinClose);
  }

  void harvest() {
    if (done) return;
    clickWait(loc + menuOption);
    unPin();
    done = true;
  }

  long getActionInterval() {
    return time(0) - lastAction;
  }

  bool match(Point p, double * match) {
    Point p2 = p + loc;
    return matchPoint(p2, match);
  }

  bool check() {
    // Return true if something changed
    if (done) return false;

    if (!match(menuCorner - (Point) { 1, 1}, blackPixel)) {
      // Menu is almost certainly not where it should be.  Die.
      done = true;
      cout << getTimeString() << " - menu not found at " << (loc + menuCorner - (Point) { 1, 1}) << endl;
      return true;
    }
    clickWait(loc);
    waitForRefresh();

    if (!match(titlePoint, blackPixel)) {
      // Menu title is missing.  Close it.
      cout << getTimeString() << " - menu title missing at " << (loc + titlePoint) << endl;
      unPin();
      return true;
    }

    if (doingSeeds) {
      return checkSeeds();
    } else {
      return checkWeeds();
    }
  }

  bool checkSeeds() {
    bool hasSeeds = match(seedPoint, matchSeed);
    bool changed = false;
    if (hadSeeds && !hasSeeds) {
      numSeeds++;
      if (numSeeds >= maxSeeds) {
	unPin();
	changed = true;
      }
    }
    if (hasSeeds && !hadSeeds) {
      doMenu();
      changed = true;
    }
    hadSeeds = hasSeeds;
    return changed;
  }

  bool checkWeeds() {
    if (match(scrollBorder, matchScrollBorder)) {
      if (getActionInterval() >= 5) {
	doMenu();
	return true;
      }
    }
    return false;
  }
};

bool willCoverPoints(Point menu, Point * pointSet, int setLen)
{
  for (int i = 0; i < setLen; i++) {
    if (menu.x <= pointSet[i].x &&
	menu.y <= pointSet[i].y &&
	menu.x + menuCorner.x >= pointSet[i].x &&
	menu.y + menuCorner.y >= pointSet[i].y) {
      return true;
    }
  }
  return false;
}

int min(int a, int b) { return (a<b) ? a : b; }

int main(int argc, char *argv[]) {
  initMacro();

  int maxArrayWidth = 7;
  int maxArrayHeight = 7;
  int maxBeds = 32;
  int runTime = 400000;

  for (int i=1; i < argc; ++i) {
    if (strcmp(argv[i], "-seeds") == 0) {
      doingSeeds = true;
    } else if (strcmp(argv[i], "-width") == 0) {
      maxArrayWidth = atoi(argv[++i]);
    } else if (strcmp(argv[i], "-height") == 0) {
      maxArrayHeight = atoi(argv[++i]);
    } else if (strcmp(argv[i], "-beds") == 0) {
      maxBeds = atoi(argv[++i]);
    } else if (strcmp(argv[i], "-runwait") == 0) {
      runTime = atoi(argv[++i]);
    } else {
      cerr << "Unknown option: " << argv[i] << endl;
      exit(1);
    }
  }

  int runPixels = tale.width * 201 / 1280; // Zoomed-in 2xf8, half coord
  Point groundPoint = { tale.width/2 - 12, tale.height/2 + 15};

  plantMenu = (Point) { 30, tale.height - 30 };
  Point westRunPoint = groundPoint + (Point) { -runPixels, 0};
  Point eastRunPoint = groundPoint + (Point) { runPixels, 0};
  Point southRunPoint = groundPoint + (Point) { 0, runPixels};
  flaxClick = groundPoint + (Point) { -10, 20 };
  const int avoidLen = 5;
  Point avoidPoints[avoidLen] = { flaxClick, plantMenu, westRunPoint, eastRunPoint, southRunPoint };

  const int arrayWidth = min(maxArrayWidth, tale.width / menuCorner.x);
  const int arrayHeight = min(maxArrayHeight, tale.height / menuCorner.y);
  maxBeds = min(maxBeds, arrayWidth * arrayHeight);
  FlaxMenu * menus[maxBeds];
  int menuIndex = 0;

  int x = 0;
  int y = 0; // x,y = positions of flax beds
  // i,j = positions of menus in window
  for (int j = 0; j < arrayHeight && menuIndex < maxBeds; j++) {
    for (int i = 0; i < arrayWidth && menuIndex < maxBeds; i++) {
      // Plant a flax bed if its menu won't get in the way
      Point proposed = (Point) { i*menuCorner.x, j*menuCorner.y };
      if (willCoverPoints(proposed, avoidPoints, avoidLen)) {
	continue;
      }
      // Run to the bed location
      Point runPoint;
      bool runNeeded = (i > 0 || j > 0);
      
      if (runNeeded) {
	if (y % 2 == 0) {
	  // Heading east
	  if (x >= arrayWidth - 1) {
	    runPoint = southRunPoint;
	    y++;
	  } else {
	    runPoint = eastRunPoint;
	    x++;
	  }
	} else {
	  // Heading west
	  if (x <= 0) {
	    runPoint = southRunPoint;
	    y++;
	  } else {
	    runPoint = westRunPoint;
	    x--;
	  }
	}
	leftClickAt(runPoint);
	usleep(runTime);
      }
      menus[menuIndex++] = new FlaxMenu(proposed);
    }
  }

  int numMenus = menuIndex;
  int numActiveMenus = numMenus;
  while (numActiveMenus > 0) {
    numActiveMenus = 0;
    for (int i = 0; i < numMenus; i++) {
      if (menus[i]->done) continue;
      menus[i]->check();
      numActiveMenus++;
    }
    sleep(1);
  }

  endMacro();
  return 0;
}
